require 'active_support/concern'

module Magma
  module Common
    extend ActiveSupport::Concern

    class_methods do
      def call(*args)
        new(*args).call
      end
    end

    def handle_output!(output, options)
      output!(output) if options[:print]
      overwrite!(options[:infile], output) if options[:overwrite]
      save!(options[:outfile], output) if options[:outfile]
      output
    end

    private

    def output!(output)
      puts output
    end

    def overwrite!(infile, output)
      raise('No file to overwrite') unless infile
      File.write infile, output
    end

    def save!(outfile, output)
      raise('No output file specified') unless outfile
      File.write outfile, output
    end
  end
end
