require 'active_support/core_ext/hash/indifferent_access'
require 'active_support/core_ext/hash/deep_merge'

module Magma::Config
  class MLT < ActiveSupport::HashWithIndifferentAccess
    VERSION = 1
    ALLOWED_KEYS = %w[
      assets
      globals
      scopes
      type
      version
    ].freeze

    attr_reader :data

    def initialize(hash = {})
      super({
        globals: {},
        scopes: {},
        type: 'mlt',
        version: VERSION,
      }.merge(hash))
      validate!
    end

    def globals
      self[:globals]
    end

    def operations
      raise 'needs to be resolved' unless resolved?
      data.select { |key, _value| key[0] == '$' }
    end

    def resolve(context = {})
      @data = context.reduce(globals.clone) do |res, (key, value)|
        scope = scopes.fetch(key, {}).fetch(value, nil)
        res.tap { |r| r.deep_merge!(scope) if scope }
      end
    end

    def resolved?
      data && true
    end

    def scopes
      self[:scopes]
    end

    def transform(doc)
      raise 'needs to be resolved' unless resolved?
      operations.each do |key, obj|
        # Skip if not prefixed with $
        leader = key[0]
        next unless leader == '$'

        # Select node set
        nodeset = doc.css(key[1..-1])

        case obj
        when Hash
          obj.each do |name, value|
            if value
              nodeset.attr(name, value)
            else
              nodeset.remove_attr(name)
            end
          end
        else
          text = obj.to_s
          nodeset.each { |node| node.content = text }
        end
      end
      doc
    end

    def variables
      raise 'needs to be resolved' unless resolved?
      data.reject { |key, _value| key[0] == '$' }
    end

    private

    def validate!
      extras = keys - ALLOWED_KEYS
      raise "extra keys: #{extras}" unless extras.empty?
      raise 'not type: mlt' unless self[:type] == 'mlt'
    end
  end
end
