require 'magma/common'
require 'magma/utils'
require 'nokogiri'
require 'securerandom'

module Magma
  class Preparer
    include Magma::Common
    include Magma::Utils

    def initialize(infile, options = {})
      @infile = infile
      @options = options
    end

    def call
      expand_resource_paths!
      handle_output! xml, options.merge(infile: infile)
      xml
    end

    private

    attr_accessor :infile, :options

    def doc
      @doc ||= Nokogiri::XML(source)
    end

    def log(message)
      # Don't print the message if we're out-putting to STDOUT
      puts message unless options[:print]
    end

    def resource_path
      return Dir.pwd unless infile
      File.dirname File.expand_path(infile)
    end

    def expand_resource_paths!
      map = Hash[doc.xpath('//producer').map do |producer|
        id = producer.get_attribute 'id'
        [id, producer]
      end.compact]
      return if map.empty?

      doc.traverse do |node|
        id = node.get_attribute 'id'
        case node.name
        when 'producer'
          if map[id]
            node.search('property[name="resource"]').each do |property|
              if File.file?(File.join(resource_path, property.content))
                log "Expanding resource '#{resource_path}/#{property.content}'"
                property.content = File.join(resource_path, property.content)
              end
            end
          end
        end
      end
    end

    def source
      @source ||= read_file_or_stdin(infile) || options[:source] || raise('Need a template')
    end

    def xml
      doc.to_xml
    end
  end
end
