require 'magma/config/mlt'
require 'magma/common'
require 'magma/utils'
require 'liquid'
require 'nokogiri'

module Magma
  class Templater
    include Magma::Common
    include Magma::Utils

    def initialize(infile, options)
      @infile   = infile
      @options  = options
    end

    def call
      pipe source, [
        :template,
        :render,
        :parse,
        :transform,
        '.to_xml',
        ->(result) { handle_output!(result, options.merge(infile: infile)) },
      ]
    end

    private

    attr_reader :infile, :options

    def config
      @config ||= Magma::Config::MLT.new(data).tap do |c|
        c.resolve(options[:context] || {})
      end
    end

    def data
      @data ||= options[:data_file].nil? ? {} : YAML.safe_load(File.read(options[:data_file]), [], [], true)
    end

    def source
      @source ||= read_file_or_stdin(infile) || options[:source] || raise('Need a template')
    end

    ### Pipeline

    # Renders a Liquid template
    def render(template)
      template.render(config.variables.deep_merge(options[:globals]), strict_variables: true).tap do
        if template.errors&.length&.positive?
          puts template.errors
          raise template.errors.map(&:to_s).join('; ')
        end
      end
    end

    # Parses a template into a DOM
    def parse(template)
      Nokogiri::XML template
    end

    # Parses a source into a template
    def template(source)
      Liquid::Template.parse(source, error_mode: :strict)
    end

    # Applies transformations to a document
    def transform(doc)
      config.transform doc
    end

    # Converts a docuemtn into XML
    def xml(doc)
      doc.to_xml
    end
  end
end
